# -*- coding: utf-8 -*-

import argparse
import logging
import os
import sys

# try:
#     from . import validate, export, gui
# except SystemError:
from pycamv import validate, export, gui, __version__


LOGGER = logging.getLogger("pycamv.main")


def _parse_args(args):
    """
    Parses arguments from a argv format.

    Parameters
    ----------
    args : list of str

    Returns
    -------
    argparse.ArgumentParser
    """
    parser = argparse.ArgumentParser(
        prog="PyCAMVerter",
        description="Aww yeah, mass specs!",
    )
    parser.add_argument(
        "-v", "--verbose",
        action="count",
        help="Increase verbosity of output.",
    )
    parser.add_argument(
        '-V', '--version',
        action="version",
        version="%(prog)s {}".format(__version__),
    )
    parser.add_argument(
        "--show_gui",
        action="store_true",
        help="Show GUI for converting files.",
    )
    parser.add_argument(
        "--raw_path",
    )
    parser.add_argument(
        "--xml_path",
    )
    parser.add_argument(
        "--scans_path",
    )
    parser.add_argument(
        "--scans",
        nargs="*", type=int,
        help="Individual scans to select for validation.",
    )
    parser.add_argument(
        "--out_path",
    )
    parser.add_argument(
        'files', nargs='*',
    )
    return parser.parse_args(args)


def main(args):
    args = _parse_args(args)

    if not args.verbose:
        level = logging.WARNING
    elif args.verbose in [1]:
        level = logging.INFO
    elif args.verbose >= 2:
        level = logging.DEBUG

    logging.basicConfig(
        # filename="pycamv.log",
        level=level,
    )

    LOGGER.debug(sys.argv)
    LOGGER.debug(args)

    if args.show_gui:
        gui.run_gui(args)
    else:
        if args.xml_path is None:
            xmls = [i for i in args.files if i.lower().endswith(".xml")]

            if len(xmls) == 1:
                args.xml_path = xmls[0]

        if args.raw_path is None:
            raws = [i for i in args.files if i.lower().endswith(".raw")]

            if len(raws) == 1:
                args.raw_path = raws[0]

        if args.scans_path is None:
            scans = [i for i in args.files if i.lower().endswith(".xlsx")]

            if len(scans) == 1:
                args.scans_path = scans[0]

        if (
            args.xml_path is None or
            args.raw_path is None
        ):
            raise Exception(
                "Missing either input xml / raw paths"
            )

        options, peak_hits, scan_mapping, precursor_windows, label_windows = (
            validate.validate_spectra(
                xml_path=args.xml_path,
                raw_path=args.raw_path,
                scans_path=args.scans_path,
                scan_list=args.scans,
            )
        )

        if args.out_path is None:
            args.out_path = os.path.splitext(args.raw_path)[0] + ".camv.gz"

        export.export_to_camv(
            args.out_path,
            peak_hits, scan_mapping, precursor_windows, label_windows,
        )

if __name__ == "__main__":
    # is_frozen_executable = getattr(sys, u'frozen', False)
    #
    # if is_frozen_executable:
    #     os.chdir(os.path.dirname(sys.executable))
    # else:
    #     os.chdir(os.path.join(os.path.dirname(__file__), u'..'))

    try:
        main(sys.argv[1:])
    except Exception as e:
        LOGGER.error("PyCAMV Converter has crashed!", exc_info=True)



"""Utility functions used in other modules."""

from __future__ import absolute_import, division

import copy
from collections import OrderedDict, Callable
import itertools
import math


def nCr(n, r):
    f = math.factorial
    return f(n) / f(r) / f(n - r)


class LenGen(object):
    def __init__(self, gen, length):
        self.gen = gen
        self.length = length

    def __call__(self):
        return itertools.islice(self.gen(), self.length)

    def __iter__(self):
        return self.gen

    def __len__(self):
        return self.length


class DefaultOrderedDict(OrderedDict):
    # Source: http://stackoverflow.com/a/6190500/562769
    def __init__(self, default_factory=None, *a, **kw):
        if (
            default_factory is not None and
            not isinstance(default_factory, Callable)
        ):
            raise TypeError("first argument must be callable")

        OrderedDict.__init__(self, *a, **kw)
        self.default_factory = default_factory

    def __getitem__(self, key):
        try:
            return OrderedDict.__getitem__(self, key)
        except KeyError:
            return self.__missing__(key)

    def __missing__(self, key):
        if self.default_factory is None:
            raise KeyError(key)

        self[key] = value = self.default_factory()

        return value

    def __reduce__(self):
        if self.default_factory is None:
            args = tuple()
        else:
            args = self.default_factory,

        return type(self), args, None, None, self.items()

    def copy(self):
        return self.__copy__()

    def __copy__(self):
        return type(self)(
            self.default_factory,
            self,
        )

    def __deepcopy__(self, memo):
        return type(self)(
            self.default_factory,
            copy.deepcopy(self.items()),
        )

    def __repr__(self):
        return "OrderedDefaultDict({}, {})".format(
            self.default_factory,
            OrderedDict.__repr__(self),
        )



"""
Provides functionality for exporting processed data to .camv files (JSON
format).
"""

from __future__ import absolute_import, division

from collections import OrderedDict
import gzip
import json
import re

from .utils import DefaultOrderedDict
from . import ms_labels


RE_PROTEIN = re.compile(r"([A-Za-z0-9\(\)\[\]\\/\',\. \-\+]+) OS=")

RE_B_Y_IONS = re.compile(r"([abcxyz]_\{[0-9]+\})\^\{\+\}(.*)")
SUPERSCRIPT_UNICODE_START = ord(u"\u2070")
SUBSCRIPT_UNICODE_START = ord(u'\u2080')
SCRIPT_MAPPING = {
    str(i): i
    for i in range(10)
}
SCRIPT_MAPPING["+"] = 10
SCRIPT_MAPPING["-"] = 11
SCRIPT_MAPPING["("] = 12
SCRIPT_MAPPING[")"] = 13


def _rewrite_ion_name(name):
    m = RE_B_Y_IONS.match(name)

    if m:
        name = "".join(m.group(1, 2))

    ret = ""
    sup, sub = False, False
    paren = 0

    for char in name:
        if char in "^_{}":
            if char == "^":
                sup, sub = True, False
            elif char == "_":
                sup, sub = False, True
            elif char == "}":
                sup, sub = False, False
                paren -= 1
            elif char == "{":
                paren += 1
            continue

        if sup:
            if char == "1":
                ret += u"\u00B9"
            elif char == "2":
                ret += u"\u00B2"
            elif char == "3":
                ret += u"\u00B3"
            else:
                ret += chr(SUPERSCRIPT_UNICODE_START + SCRIPT_MAPPING[char])
        elif sub:
            ret += chr(SUBSCRIPT_UNICODE_START + SCRIPT_MAPPING[char])
        else:
            ret += char

        if sup or sub:
            if not paren:
                sup, sub = False, False

    return ret


def _peaks_to_dict(peaks):
    return [
        OrderedDict([
            ("mz", mz),
            ("into", i),
        ])
        for mz, i in peaks
    ]


def _join_seq_mods(seq, mods):
    return tuple(zip(["N-term"] + list(seq) + ["C-term"], mods))


def _extract_pep_seq(sequence):
    return "".join(
        letter
        for letter, _ in sequence
        if letter not in ["N-term", "C-term"]
    )


def _extract_mods(sequence):
    return tuple(
        tuple(mods)
        for _, mods in sequence
    )


def _get_mods_description(pep_seq, mod_state):
    return ("+ " if mod_state else "") + " - ".join(
        "{} {}{}".format(count, mod[0].lower(), "".join(letters))
        for count, mod, letters in mod_state
    )


def _mod_positions(mods):
    return [
        index
        for index, mod in enumerate(mods)
        if mod
    ]


def _pep_mod_name(pep_seq, mods):
    return "".join(
        letter.lower() if mods else letter.upper()
        for letter, mods in zip(pep_seq, mods[1:-1])
    )


def _get_labels_mz(query):
    def _within(val, bounds):
        return val >= bounds[0] and val <= bounds[1]
    return [
        mz
        for mod in set(query.get_label_mods)
        for mz in ms_labels.LABEL_MASSES.get(mod, [])
        if _within(mz, ms_labels.LABEL_MZ_WINDOW.get(mod, [mz, mz]))
    ]


def export_to_camv(
    out_path, peak_hits, scan_mapping,
    precursor_windows, label_windows,
):
    """
    Export data to .camv file.

    Parameters
    ----------
    out_path : str
    peak_hits
    scan_mapping
    precursor_windows
    label_windows

    Returns
    -------
    data : dict
        Dictionary of data written to file.

    Example
    -------
    >>> from pycamv import export, validate
    >> name = "2016-06-27-CKX13-pY-NTA-elute-pre67-col77"
    >>> options, peak_hits, scan_mapping, precursor_windows, label_windows = \
    ...     validate.validate_spectra(name)
    >>> json_out = export.export_to_camv(
    ...     "{}.camv".format(name),
    ...     peak_hits, scan_mapping, precursor_windows, label_windows,
    ... )
    """
    ###
    # Mappings between proteins, peptides, modifications, queries, and peaks
    ###
    # Mapping for protein -> peptides
    prot_dict = DefaultOrderedDict(set)

    for query, _ in peak_hits.keys():
        prot_dict[RE_PROTEIN.match(query.protein).group(1)].add(query.pep_seq)

    pep_dict = DefaultOrderedDict(set)

    for query, _ in peak_hits.keys():
        pep_dict[query.pep_seq].add(tuple(query.pep_var_mods))

    mod_states_dict = DefaultOrderedDict(set)

    for query, seq in peak_hits.keys():
        mod_states_dict[query.pep_seq, tuple(query.pep_var_mods)].add(
            _extract_mods(seq)
        )

    mod_query_dict = DefaultOrderedDict(set)

    for query, seq in peak_hits.keys():
        mod_query_dict[query.pep_seq, tuple(query.pep_var_mods)].add(query)

    # Mapping for modifications -> queries
    mods_dict = DefaultOrderedDict(list)

    for query, seq in peak_hits.keys():
        mods_dict[query.pep_seq, _extract_mods(seq)].append(query)

    # Mapping for queries -> sequence + modifications
    query_dict = DefaultOrderedDict(list)

    for query, seq in peak_hits.keys():
        query_dict[query].append((query.pep_seq, _extract_mods(seq)))

    ###
    # Pre-calculate IDs for later reference
    ###
    # Protein IDs
    prot_index = {
        prot_name: index
        for index, prot_name in enumerate(sorted(prot_dict.keys()))
    }

    # Peptide IDs
    pep_index = {
        (pep_seq, mod_states): index
        for peptides in prot_dict.values()
        for index, (pep_seq, mod_states) in enumerate(
            (pep_seq, mod_states)
            for pep_seq in peptides
            for mod_states in pep_dict[pep_seq]
        )
    }

    # Peptide Data IDs
    pep_data_index = {
        pep_seq: index
        for index, pep_seq in enumerate(
            sorted(
                pep_seq
                for peptides in prot_dict.values()
                for pep_seq in peptides
            )
        )
    }

    # Mod State IDs
    mod_state_index = {
        (pep_seq, mod_state): index
        for pep_seq, mod_states in pep_dict.items()
        for index, mod_state in enumerate(mod_states)
    }

    # Modification IDs
    mod_index = {
        (pep_seq, mod): index
        for (pep_seq, _), mods in mod_states_dict.items()
        for index, mod in enumerate(mods)
    }

    # Scan IDs
    scan_index = {
        query: index
        for pep_seq, mod_states in pep_dict.items()
        for mod_state in mod_states
        for index, query in enumerate(
            sorted(
                mod_query_dict[pep_seq, mod_state],
                key=lambda x: x.scan,
            )
        )
    }

    # Peak Match IDs
    match_index = {
        (pep_seq, mods, name): index
        for (pep_seq, mods), queries in mods_dict.items()
        for index, name in enumerate(
            name
            for name in sorted(
                set(
                    name
                    for query in queries
                    for peak_hit in peak_hits[
                        query, _join_seq_mods(pep_seq, mods)
                    ]
                    if peak_hit.match_list
                    for name in peak_hit.match_list.keys()
                )
            )
        )
    }

    ###
    # Individual data parsing functions
    ###
    def _gen_match_data(pep_seq, mods):
        """
        Generate a list of all potential peak matches
        """
        visited = set()
        queries = mods_dict[pep_seq, mods]

        for query in queries:
            for peak_hit in peak_hits[query, _join_seq_mods(pep_seq, mods)]:
                if not peak_hit.match_list:
                    continue

                for name, (mz, _) in peak_hit.match_list.items():
                    if name in visited:
                        continue

                    name_split = re.split(r"[_\^]", name)
                    name_split = [i.strip("{}") for i in name_split]

                    ion_type, ion_pos = None, None

                    if len(name_split) > 1:
                        if name_split[0] in "abc":
                            ion_type, ion_pos = "b", int(name_split[1])
                        elif name_split[0] in "xyz":
                            ion_type, ion_pos = "y", int(name_split[1])

                    yield OrderedDict([
                        ("id", match_index[pep_seq, mods, name]),
                        ("mz", mz),
                        ("name", _rewrite_ion_name(name)),
                        ("ionType", ion_type),
                        ("ionPosition", ion_pos),
                    ])

                    visited.add(name)

    def _get_match_data(pep_seq, mods):
        return sorted(
            _gen_match_data(pep_seq, mods),
            key=lambda x: x["id"],
        )

    def _get_mod_data(pep_seq, mods):
        return [
            OrderedDict([
                ("id", mod_index[pep_seq, mod]),
                ("position", _mod_positions(mod)),
                ("name", _pep_mod_name(pep_seq, mod)),
                ("matchData", _get_match_data(pep_seq, mod)),
            ])
            for mod in sorted(
                mods,
                key=lambda x: mod_index[pep_seq, x],
            )
        ]

    def _get_mod_states(pep_seq, mod_states):
        return [
            OrderedDict([
                ("id", mod_state_index[pep_seq, mod_state]),
                ("modDesc", _get_mods_description(pep_seq, mod_state)),
                (
                    "mods",
                    _get_mod_data(
                        pep_seq,
                        mod_states_dict[pep_seq, mod_state],
                    ),
                ),
            ])
            for mod_state in sorted(
                mod_states,
                key=lambda x: mod_state_index[pep_seq, x],
            )
        ]

    def _get_peptide_data():
        """
        Return all information mapping (modified) peptides to their sequence,
        descriptions, ion fragmentation patterns, etc.
        """
        return [
            OrderedDict([
                ("id", pep_data_index[pep_seq]),
                ("peptideSequence", pep_seq),
                ("modificationStates", _get_mod_states(pep_seq, mod_states)),
            ])
            for pep_seq, mod_states in sorted(
                pep_dict.items(),
                key=lambda x: pep_data_index[x[0]],
            )
        ]

    def _get_default_choice_data(pep_seq, mod_state):
        return [
            OrderedDict([
                ("modsId", mod_index[pep_seq, mod]),
                ("state", None),  # null
            ])
            for mod in sorted(
                mod_states_dict[pep_seq, mod_state],
                key=lambda x: mod_index[pep_seq, x],
            )
        ]

    def _get_matches(peak_index, query):
        return [
            OrderedDict([
                ("modsId", mod_index[seq, mods]),
                (
                    "matchId",
                    match_index.get(
                        (
                            seq,
                            mods,
                            peak_hits[
                                query,
                                _join_seq_mods(seq, mods),
                            ][peak_index].name,
                        ),
                        None,
                    ),
                ),
            ])
            for seq, mods in sorted(
                query_dict[query],
                key=lambda x: mod_index[x],
            )
        ]

    def _get_scan_assignments(query, seq):
        mod = query_dict[query][0][1]

        return [
            OrderedDict([
                ("mz", peak_hit.mz),
                ("into", peak_hit.intensity),
                ("matchInfo", _get_matches(peak_index, query)),
            ])
            for peak_index, peak_hit in sorted(
                enumerate(peak_hits[query, _join_seq_mods(seq, mod)]),
                key=lambda x: x[1].mz,
            )
        ]

    def _get_scans(pep_seq, mod_state):
        """
        Return information on individual scans, including peaks, precursor
        ions, and peptide modification assignments.
        """
        return [
            OrderedDict([
                ("fileName", query.filename),
                ("scanNumber", query.scan),
                ("scanId", scan_index[query]),
                ("chargeState", query.pep_exp_z),
                (
                    "precursorScanData",
                    _peaks_to_dict(precursor_windows[query]),
                ),
                ("precursorMz", query.pep_exp_mz),
                (
                    "precursorIsolationWindow",
                    scan_mapping[query].window_offset,
                ),
                ("quantScanData", _peaks_to_dict(label_windows[query])),
                ("quantMz", _get_labels_mz(query)),
                (
                    "choiceData",
                    _get_default_choice_data(pep_seq, mod_state),
                ),
                (
                    "scanData",
                    _get_scan_assignments(query, pep_seq),
                ),
            ])
            for query in sorted(
                mod_query_dict[pep_seq, mod_state],
                key=lambda x: scan_index[x],
            )
        ]

    def _get_peptide_scan_data(peptides):
        """
        Map peptides to their data IDs, scans, and candidate modification
        states.
        """
        return [
            OrderedDict([
                ("peptideId", pep_index[pep_seq, mod_state]),
                ("peptideDataId", pep_data_index[pep_seq]),
                ("modificationStateId", mod_state_index[pep_seq, mod_state]),
                ("scans", _get_scans(pep_seq, mod_state)),
            ])
            for pep_seq, mod_state in sorted(
                (
                    (pep_seq, mod_state)
                    for pep_seq in peptides
                    for mod_state in pep_dict[pep_seq]
                ),
                key=lambda x: pep_index[x],
            )
        ]

    def _get_scan_data():
        """
        Return all information mapping proteins / peptides to their scans and
        a list of candidate modification patterns.
        """
        return [
            OrderedDict([
                ("proteinName", prot_name),
                ("proteinId", prot_index[prot_name]),
                ("peptides", _get_peptide_scan_data(peptides)),
            ])
            for prot_name, peptides in sorted(
                prot_dict.items(),
                key=lambda x: prot_index[x[0]],
            )
        ]

    data = OrderedDict([
        ("peptideData", _get_peptide_data()),
        ("scanData", _get_scan_data()),
    ])

    if out_path.endswith(".gz"):
        with gzip.GzipFile(out_path, 'w') as f:
            f.write(json.dumps(data, indent=None).encode("utf-8"))
    else:
        with open(out_path, "w") as f:
            json.dump(data, f, indent=None)

    return data



"""
This module provides functionality for interacting with CAMV.

Currently limited to importing and outputing scan lists.
"""

# Built-ins
from __future__ import absolute_import, division

from collections import OrderedDict
from functools import partial
import logging
import multiprocessing
import re
import tempfile

from openpyxl import load_workbook

from . import compare, fragments, gen_sequences, mascot, ms_labels, scans
from .utils import LenGen


LOGGER = logging.getLogger("pycamv.validate")


class SearchOptions:
    """
    Contains options used to search a data set in MASCOT.

    Attributes
    ----------
    label_type : tuple of (str, int)
    """
    def __init__(self, fixed_mods, var_mods):
        self.label_type = (None, 0)

        for mod in fixed_mods:
            mod = re.sub(r"([\w-]+) \([\w-]+\)", r"\1", mod)
            num = ms_labels.LABEL_NUMBERS.get(mod, 0)

            if num > 0:
                self.label_type = ("Fixed", num)

        for mod in var_mods:
            mod = re.sub(r"([\w-]+) \([\w-]+\)", r"\1", mod)
            num = ms_labels.LABEL_NUMBERS.get(mod, 0)

            if num > 0:
                self.label_type = ("Variable", num)

        # TODO: Parse out SILAC, C-mod, phospho, etc


def _remap_pst(pep_mods):
    return [
        (
            count,
            mod,
            letters + (
                ("Y",)
                if (mod == "Phospho" and set(letters) == set(["S", "T"])) else
                ()
            ),
        )
        for count, mod, letters in pep_mods
    ]


def load_scan_list(scans_path):
    wb = load_workbook(scans_path)
    ws = wb.active
    return [row[0].value for row in ws.iter_rows()]


def _map_seq(kv):
    pep_query = kv
    return (
        pep_query,
        tuple(
            gen_sequences.gen_possible_seq(
                pep_query.pep_seq,
                pep_query.pep_mods,
            )
        ),
    )


def _map_frag(kv):
    pep_query, sequence = kv
    return (
        (pep_query, tuple(sequence)),
        fragments.fragment_ions(
            sequence, pep_query.pep_exp_z,
        ),
    )


def _map_compare(kv, scan_mapping):
    key, val = kv
    pep_query, sequence = key
    frag_ions, scan = val

    peaks = compare.compare_spectra(
        scan,
        frag_ions,
        pep_query.pep_exp_z,
        scans.c13_num(pep_query, scan_mapping[pep_query]),
        tol=compare.COLLISION_TOLS[scan_mapping[pep_query].collision_type],
    )

    del scan

    return key, peaks


def validate_spectra(
    xml_path,
    raw_path,
    scans_path=None,
    scan_list=None,
    cpu_count=None,
):
    """
    Generate CAMV web page for validating spectra.

    Parameters
    ----------
    xml_path : str
    raw_path : str
    scans_path : str, optional
    scan_list : list of int, optional
    cpu_count : int, optional

    Returns
    -------
    options : :class:`pycamv.validate.SearchOptions`
    peak_hits : dict of (tuple of :class:`pycamv.mascot.PeptideQuery`, list),
    list of :class:`pycamv.compare.PeptideHit`
        Dictionary mapping peptide queries and their sequences to peptide hits.
    scan_mapping : OrderedDict of :class:`pycamv.mascot.PeptideQuery`,
    :class:`pycamv.scans.ScanQuery`
        Dictionary mapping peptide queries to their scan data, including
        collision type and size of isolation window.
    precursor_windows : dict of :class:`pycamv.mascot.PeptideQuery`, list
        Dictionary mapping peptide queries to peak lists for precursor scans.
    label_windows : dict of :class:`pycamv.mascot.PeptideQuery`, list
        Dictionary mapping peptide queries to peak lists for quantification
        channels.
    """
    if cpu_count is None:
        try:
            cpu_count = multiprocessing.cpu_count()
        except NotImplementedError:
            cpu_count = 2

    LOGGER.debug("Validating data set with {} cpus".format(cpu_count))

    if scan_list is None:
        scan_list = []

    if scans_path is not None:
        scan_list += load_scan_list(scans_path)

    # Read MASCOT xml file
    fixed_mods, var_mods, pep_queries = mascot.read_mascot_xml(xml_path)

    # Optionally filter queries using a scan list
    if scan_list:
        pep_queries = [
            query
            for query in pep_queries
            if query.scan in scan_list
        ]

    # Remap pST -> pSTY
    LOGGER.info("Remapping pST -> pSTY")
    for pep_query in pep_queries:
        pep_query.pep_var_mods = _remap_pst(pep_query.pep_var_mods)
        pep_query.pep_fixed_mods = _remap_pst(pep_query.pep_fixed_mods)

    # Extract MASCOT search options
    options = SearchOptions(fixed_mods, var_mods)

    # Remove peptides with an excess of modification combinations

    # Get scan data from RAW file
    out_dir = tempfile.mkdtemp()

    LOGGER.info("Getting scan data.")

    scan_queries, ms_two_data, ms_data = scans.get_scan_data(
        raw_path, pep_queries, out_dir,
    )

    scan_mapping = OrderedDict(
        (pep_query, scan_query)
        for pep_query, scan_query in zip(pep_queries, scan_queries)
    )

    # Generate sequences
    LOGGER.info(
        "Generating all possible sequence-modification combinations."
    )
    pool = multiprocessing.Pool(processes=cpu_count)
    sequence_mapping = OrderedDict(
        pool.map(_map_seq, pep_queries)
    )

    LOGGER.info(
        "Generating fragment ions for {} queries.".format(
            len(sequence_mapping),
        )
    )
    fragment_mapping = OrderedDict(
        pool.map(
            _map_frag,
            (
                (pep_query, sequence)
                for pep_query, sequences in sequence_mapping.items()
                for sequence in sequences
            ),
        )
    )

    LOGGER.info(
        "Comparing predicted to actual peaks for {} spectra.".format(
            len(fragment_mapping),
        )
    )

    def _imap_compare(kv):
        key, val = kv
        return (key, (val, ms_two_data[key[0].scan].deRef()))

    peak_hits = dict(
        pool.map(
            partial(_map_compare, scan_mapping=scan_mapping),
            LenGen(
                (
                    (key, (val, ms_two_data[key[0].scan].deRef()))
                    for key, val in fragment_mapping.items()
                ),
                len(fragment_mapping),
            ),
            cpu_count * 4,
        )
    )

    del pool

    # XXX: Determine SILAC precursor masses?

    LOGGER.info("Collecting precursor ion peaks.")
    precursor_windows = OrderedDict(
        zip(
            pep_queries,
            scans.get_precursor_peak_window(scan_queries, ms_data)
        )
    )

    LOGGER.info("Collecting peptide label peaks.")
    label_windows = OrderedDict(
        zip(
            pep_queries,
            scans.get_label_peak_window(pep_queries, ms_two_data)
        )
    )

    # XXX: Remove precursor contaminated scans from validation list?

    # Check each assignment to each scan

    # Output data

    # shutil.rmtree(out_dir)

    return options, peak_hits, scan_mapping, precursor_windows, label_windows
